#!/usr/bin/python3
import glob
import subprocess

kernel_c_sources = glob.glob("kernel/*.c") + glob.glob("kernel/*/*.c") + glob.glob("kernel/*/*/*.c")
kernel_s_sources = glob.glob("kernel/*.S")

cflags = "-O2 -std=c99 -finline-functions -ffreestanding -Wall -Wextra -Wno-unused-function -Wno-unused-parameter -Wno-format -pedantic -fno-omit-frame-pointer -D_KERNEL_ -DKERNEL_GIT_TAG=master"

for i in kernel_c_sources:
    cmd = "gcc {cflags} -nostdlib -g -c -o {obj} {src}".format(
        cflags=cflags,
        obj=i.replace('.c','.o'),
        src=i
    )
    print(cmd)
    subprocess.run(cmd, shell=True)

for i in kernel_s_sources:
    cmd = "as -o {obj} {src}".format(
        obj=i.replace('.S','.o'),
        src=i
    )
    print(cmd)
    subprocess.run(cmd,shell=True)

objects = [x.replace('.c','.o') for x in kernel_c_sources] + [x.replace('.S','.o') for x in kernel_s_sources if not x.endswith('symbols.S')]

cmd = "gcc -T kernel/link.ld {cflags} -nostdlib -o .toaruos-kernel {objects} -lgcc".format(
    cflags=cflags,
    objects=' '.join(objects),
)
print(cmd)
subprocess.run(cmd, shell=True)

cmd = "nm -g .toaruos-kernel | generate_symbols.py > kernel/symbols.S"
print(cmd)
subprocess.run(cmd,shell=True)

subprocess.run("rm .toaruos-kernel", shell=True)

cmd = "as -o kernel/symbols.o kernel/symbols.S"
print(cmd)
subprocess.run(cmd,shell=True)

cmd = "gcc -T kernel/link.ld {cflags} -nostdlib -o kernel.elf {objects} kernel/symbols.o -lgcc".format(
    cflags=cflags,
    objects=' '.join(objects),
)
print(cmd)
subprocess.run(cmd, shell=True)

